function [classes, cert] = bayes_class(varargin)
//-----------------------------------------------------------------------  
//function [classes, cert] = bayes_class(varargin)
//
// implementation of bayes classification of multi-dimensional 
// vectors in case of a c-class problem with class conditional  
// probability distributions assumed to be normal distributions 
// definined by the statistical parameters p1,m1,K1,p2,m2,K2,... 
// it's multi-dimensional classification(more advanced to
// bayes_class_2d)
  
// input:     varargin    (but should contain arguments:  x & atleast
//                        one set of p, m & K)
//                   x    n1 - times n2 - real matrix consisting of n
//                        points to be classified
//         p1, p2, ...    prior probabilities of the c classes (sum == 1)
//         m1, m2, ...    mean vectors of the c classes
//         K1, K2, ...    covariance matrices of the c  distributions
//
// output:     classes    n2-vector of estimated class memberships  
//                cert    n2-vector of certainty measures for the 
//                        classifications, in this case P(w_cl | x)
//
//-----------------------------------------------------------------------  
//   $RCSfile: bayes_class.sci,v $
//  $Revision: 1.1 $
//      $Date: 2002/09/10 15:15:36 $
//  Copyright: IIF-LMB $Author: poluru $
//
//-----------------------------------------------------------------------
//  $Log: bayes_class.sci,v $
//  Revision 1.1  2002/09/10 15:15:36  poluru
//  "macros" directory structure is created
//
//-----------------------------------------------------------------------

  S = length(varargin)-1;
    
  if modulo(S,3) ~= 0 , 
    error ('wrong number of input arguments!');
  end;
   
  x=varargin(1);
  n1=size(x,1); n2= size(x,2); 
  p=list(); m=list();M=list(); K=list(); 
  i=2; j=0; psum=0,
  
  if S==-1,
    error('you must enter atleast the points to be classified');
  end
  
  //setting default classification data if it's not given by the
  //user. works only for 2-dimensional classification. 
  //"DATA" is the absolute path to the "data" directory. 
  if S==0,
    global DATA;
    exec(DATA+'data.sci');
    
    if size(m1,1) ~=n1,      
      error('class statistics & the data should have same dimension');     
    end;
    
    p1 = 0.5; 
    p2 = 0.5;
    p = list(p1,p2);
    M = list( repmat(m1,1,n2),repmat(m2,1,n2) );
    K = list(K1,K2);
    psum = p1+p2;  
    j = 2;
  else  
    
    while i<= length(varargin),
      j=1+(i-2)/3;
      p(j)= varargin(i); 
      m(j)= varargin(i+1);
      K(j)= varargin(i+2);
      i=i+3; psum= psum+ p(j);
     
      if p(j)<0,
	error('Error in the value of p'+string(j));
      end
      
      if size(m(j),1) ~=n1,
	if size(m(j)',1)~=n1,
	  error('Error in the size of m'+string(j));
	end
	m(j)=m(j)';
      end
      
      M(j)=repmat(m(j),1,n2);
      
      if size(K(j),1)~=n1 | size(K(j),2)~=n1,
	error('Error in the size of K'+string(j));
      end
    end
  end
  
    if psum~=1,
      error('Sum of all the p''s (probabilities) should be 1');
    end
 
  e=list();ccp=list();
  d=list();dsum=[];
  
  // calculating the values of map decision 
  // for all the sets(j sets) of classification data 
  
  for i= 1: j,    
    e(i) = -0.5 * sum(( (x-M(i))' * pinv(K(i)) )' .* (x-M(i)),'r');
    ccp(i) = 1/(2*%pi*sqrt(det(K(i))^(n1/2))) * exp(e(i));
    
    // calculate values for MAP decision:
    d(i) = p(i) * ccp(i);
    dsum=dsum+d(i);
  end
  
  // MAP decision and generation of class labels
  [maxp, classes] = max(d(:));
  
  // a-posteriori probabilities (certainty)
  //replacing the occurances of zeros by ones
  dsum(find(dsum==0))=1;
  
  cert = maxp ./ (dsum);
  
endfunction







