function [classes, cert] = polynom_class(X, p, Xlearn, labels)
//-----------------------------------------------------------------------  
// function [classes, cert] = polynom_class(X, p, Xlearn, labels)
//
// perform polynomial regression of learning data and use this
// for classification.
//
// input:    X            arbitrary sized matrix of columnwise vector
//                        which have to be classified.
//           p            polynomial degree to be used, positive integer 
//           Xlearn       Matrix of columnwise learning vectors,
//                        dimension of the vectors has to coincide with X
//           labels       vector of integer-labels (starting from 1) of
//                        the learning vectors
//
// output:   classes      vector of estimated class-numbers for each 
//                        vector in X.          
//           cert         vector of "certainties" for each class decision
//                        values between 0 and 1, simply the maximum 
//                        of the winning decision function.
//
//-----------------------------------------------------------------------  
//   $RCSfile: polynom_class.sci,v $
//  $Revision: 1.1 $
//      $Date: 2002/09/10 15:15:36 $
//  Copyright: IIF-LMB $Author: poluru $
//
//-----------------------------------------------------------------------
//  $Log: polynom_class.sci,v $
//  Revision 1.1  2002/09/10 15:15:36  poluru
//  "macros" directory structure is created
//
//-----------------------------------------------------------------------

  
  [out,in] = argn(0);
  
  if in~=4,     
    if in~=2,
      if in~=1,
	error('wrong number of input arguments.');
      end  
      p = 1; 
    end    
   
    //setting default classification data if it's not given by the
    //user. works only for 2-dimensional classification. 
    //"DATA" is the absolute path to the "data" directory. 
    global DATA;
    exec(DATA+'data.sci');
    Xlearn=V;
    labels=l;
    //clear V l
  end
  
  if size(X,1)~=size(Xlearn,1), 
  error('classification data & learning data should have to same  dimension');
  end
  
  if size(labels,2)~=size(Xlearn,2), 
  error('no of labels should be equal to the no. of  learning data points');
  end
  
  // build label-vectors
  Y = zeros(max(labels),size(Xlearn,2));
  ind = labels + (0:(length(labels)-1))*max(labels); 
  Y(ind)=ones(length(ind));
  
  // calculate regression matrix
  A = regress_matrix(p,Xlearn,Y);  
  clear Xlearn Y 
 
  // perform classification, calculate "certainties" : maximum of outputs
  cvec = polynom_regress(p,X,A);
  cert = max(cvec,'r');  
  
  cmaxvec = ones(max(labels),1)*cert; 
  diffvec = cvec-cmaxvec;
  
  clear A cmaxvec p 
  
  // calculate class-numbers
  classes = modulo(find(diffvec(:)>=0)-1, size(cvec,1))+1;
  if (length(classes)~= size(X,2))
    error("error in class-label calculation!");
  end;
  
  clear X cvec diffvec 
  
  ind = find(cert<0);
  cert(ind)= zeros(length(ind));
  
  ind = find(cert>1);
  cert(ind)= ones(length(ind));
  
  clear ind

endfunction;

  
